@@ -4,57 +4,69 @@ import logging
 import json
 import re
 import requests
+import cloudscraper
 from enum import Enum, auto
 
 from .__version__ import __version__
 
 BASE_URL = 'https://connect.garmin.com'
 SSO_URL = 'https://sso.garmin.com/sso'
-MODERN_URL = 'https://connect.garmin.com/modern'
 SIGNIN_URL = 'https://sso.garmin.com/sso/signin'
 
-
 class Garmin(object):
     """
     Object using Garmin Connect 's API-method.
     See https://connect.garmin.com/
+    or you are in mainland China
+    See https://connect.garmin.cn/
     """
-    url_user_summary = MODERN_URL + '/proxy/usersummary-service/usersummary/daily/'
-    url_user_summary_chart = MODERN_URL + \
-        '/proxy/wellness-service/wellness/dailySummaryChart/'
-    url_heartrates = MODERN_URL + '/proxy/wellness-service/wellness/dailyHeartRate/'
-    url_sleepdata = MODERN_URL + '/proxy/wellness-service/wellness/dailySleepData/'
-    url_body_composition = MODERN_URL + \
-        '/proxy/weight-service/weight/daterangesnapshot'
-    url_activities = MODERN_URL + \
-        '/proxy/activitylist-service/activities/search/activities'
-    url_activity = MODERN_URL + '/proxy/activity-service/activity/'
-    url_personal_record = MODERN_URL + '/proxy/personalrecord-service/personalrecord/'
-    url_tcx_download = MODERN_URL + "/proxy/download-service/export/tcx/activity/"
-    url_gpx_download = MODERN_URL + "/proxy/download-service/export/gpx/activity/"
-    url_kml_download = MODERN_URL + "/proxy/download-service/export/kml/activity/"
-    url_fit_download = MODERN_URL + "/proxy/download-service/files/activity/"
-    url_csv_download = MODERN_URL + "/proxy/download-service/export/csv/activity/"
-    url_device_list = MODERN_URL + '/proxy/device-service/deviceregistration/devices'
-    url_device_service = MODERN_URL + \
-        '/proxy/device-service/deviceservice/'
 
-    headers = {
-        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/79.0.3945.88 Safari/537.36',
-        'origin': 'https://sso.garmin.com'
-    }
-
-    def __init__(self, email, password):
+    def __init__(self, email, password, is_cn=False):
         """
         Init module
         """
+        global BASE_URL
+        global SSO_URL
+        global SIGNIN_URL
         self.email = email
         self.password = password
+        self.cf_req = cloudscraper.CloudScraper()
         self.req = requests.session()
         self.logger = logging.getLogger(__name__)
         self.display_name = ""
         self.full_name = ""
         self.unit_system = ""
+        self.is_cn = is_cn
+        if is_cn:
+            BASE_URL = BASE_URL.replace(".com", ".cn")
+            SSO_URL = SSO_URL.replace(".com", ".cn")
+            SIGNIN_URL = SIGNIN_URL.replace(".com", ".cn")
+
+        self.url_user_summary = BASE_URL + '/proxy/usersummary-service/usersummary/daily/'
+        self.url_user_summary_chart = BASE_URL + \
+            '/proxy/wellness-service/wellness/dailySummaryChart/'
+        self.url_heartrates = BASE_URL + '/proxy/wellness-service/wellness/dailyHeartRate/'
+        self.url_sleepdata = BASE_URL + '/proxy/wellness-service/wellness/dailySleepData/'
+        self.url_body_composition = BASE_URL + \
+            '/proxy/weight-service/weight/daterangesnapshot'
+        self.url_activities = BASE_URL + \
+            '/proxy/activitylist-service/activities/search/activities'
+        self.url_hydrationdata = BASE_URL + '/proxy/usersummary-service/usersummary/hydration/daily/'
+        self.url_activity = BASE_URL + '/proxy/activity-service/activity/'
+        self.url_personal_record = BASE_URL + '/proxy/personalrecord-service/personalrecord/'
+        self.url_tcx_download = BASE_URL + "/proxy/download-service/export/tcx/activity/"
+        self.url_gpx_download = BASE_URL + "/proxy/download-service/export/gpx/activity/"
+        self.url_kml_download = BASE_URL + "/proxy/download-service/export/kml/activity/"
+        self.url_fit_download = BASE_URL + "/proxy/download-service/files/activity/"
+        self.url_csv_download = BASE_URL + "/proxy/download-service/export/csv/activity/"
+        self.url_device_list = BASE_URL + '/proxy/device-service/deviceregistration/devices'
+        self.url_device_service = BASE_URL + \
+            '/proxy/device-service/deviceservice/'
+
+        self.headers = {
+            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/79.0.3945.88 Safari/537.36',
+            'origin': 'https://sso.garmin.com' if not self.is_cn else "https://sso.garmin.cn"
+        }
 
     def login(self):
         """
@@ -62,10 +74,10 @@ class Garmin(object):
         """
         params = {
             'webhost': BASE_URL,
-            'service': MODERN_URL,
+            'service': BASE_URL,
             'source': SIGNIN_URL,
-            'redirectAfterAccountLoginUrl': MODERN_URL,
-            'redirectAfterAccountCreationUrl': MODERN_URL,
+            'redirectAfterAccountLoginUrl': BASE_URL,
+            'redirectAfterAccountCreationUrl': BASE_URL,
             'gauthHost': SSO_URL,
             'locale': 'en_US',
             'id': 'gauth-widget',
@@ -82,6 +94,8 @@ class Garmin(object):
             'embedWidget': 'false',
             'generateExtraServiceTicket': 'false'
         }
+        if self.is_cn:
+            params['cssUrl'] = 'https://static.garmincdn.cn/cn.garmin.connect/ui/css/gauth-custom-v1.2-min.css'
 
         data = {
             'username': self.email,
@@ -95,12 +109,15 @@ class Garmin(object):
         self.logger.debug(
             "Login to Garmin Connect using POST url %s", SIGNIN_URL)
         try:
-            response = self.req.post(
+            response = self.cf_req.get(
+                SIGNIN_URL, headers=self.headers, params=params)
+
+            response = self.cf_req.post(
                 SIGNIN_URL, headers=self.headers, params=params, data=data)
             if response.status_code == 429:
                 raise GarminConnectTooManyRequestsError("Too many requests")
-
             response.raise_for_status()
+            self.req.cookies = self.cf_req.cookies
             self.logger.debug("Login response code %s", response.status_code)
         except requests.exceptions.HTTPError as err:
             raise GarminConnectConnectionError("Error connecting") from err
@@ -323,9 +340,9 @@ class Garmin(object):
 
     def get_excercise_sets(self, activity_id):
         activity_id = str(activity_id)
-        exercisesetsurl = f"{self.url_activity}{activity_id}/exerciseSets"
+        exercisesetsurl = f"{self.url_activity}{activity_id}"
         self.logger.debug(
-            f"Fetching exercise sets for activity_id {activity_id}")
+            f"Fetching excercise sets for activity_id {activity_id}")
 
         return self.fetch_data(exercisesetsurl)
 
@@ -404,6 +421,15 @@ class Garmin(object):
             "Fetching device last used with url %s", device_last_used_url)
         return self.fetch_data(device_last_used_url)
 
+    def get_hydration_data(self, cdate):   # cDate = 'YYYY-mm-dd'
+        """
+        Fetch available hydration data
+        """
+        hydration_url = self.url_hydrationdata + cdate
+        self.logger.debug("Fetching hydration data with url %s", hydration_url)
+
+        return self.fetch_data(hydration_url)
+
     class ActivityDownloadFormat(Enum):
         ORIGINAL = auto()
         TCX = auto()
